@@ -1010,9 +1010,9 @@ f_winrestcmd(typval_T *argvars UNUSED, typval_T *rettv)
     ga_init2(&ga, (int)sizeof(char), 70);
     FOR_ALL_WINDOWS(wp)
     {
-	sprintf((char *)buf, "%dresize %d|", winnr, wp->w_height);
+	sprintf((char *)buf, ":%dresize %d|", winnr, wp->w_height);
 	ga_concat(&ga, buf);
-	sprintf((char *)buf, "vert %dresize %d|", winnr, wp->w_width);
+	sprintf((char *)buf, "vert :%dresize %d|", winnr, wp->w_width);
 	ga_concat(&ga, buf);
 	++winnr;
     }
