@@ -750,6 +750,258 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    2253,
+/**/
+    2252,
+/**/
+    2251,
+/**/
+    2250,
+/**/
+    2249,
+/**/
+    2248,
+/**/
+    2247,
+/**/
+    2246,
+/**/
+    2245,
+/**/
+    2244,
+/**/
+    2243,
+/**/
+    2242,
+/**/
+    2241,
+/**/
+    2240,
+/**/
+    2239,
+/**/
+    2238,
+/**/
+    2237,
+/**/
+    2236,
+/**/
+    2235,
+/**/
+    2234,
+/**/
+    2233,
+/**/
+    2232,
+/**/
+    2231,
+/**/
+    2230,
+/**/
+    2229,
+/**/
+    2228,
+/**/
+    2227,
+/**/
+    2226,
+/**/
+    2225,
+/**/
+    2224,
+/**/
+    2223,
+/**/
+    2222,
+/**/
+    2221,
+/**/
+    2220,
+/**/
+    2219,
+/**/
+    2218,
+/**/
+    2217,
+/**/
+    2216,
+/**/
+    2215,
+/**/
+    2214,
+/**/
+    2213,
+/**/
+    2212,
+/**/
+    2211,
+/**/
+    2210,
+/**/
+    2209,
+/**/
+    2208,
+/**/
+    2207,
+/**/
+    2206,
+/**/
+    2205,
+/**/
+    2204,
+/**/
+    2203,
+/**/
+    2202,
+/**/
+    2201,
+/**/
+    2200,
+/**/
+    2199,
+/**/
+    2198,
+/**/
+    2197,
+/**/
+    2196,
+/**/
+    2195,
+/**/
+    2194,
+/**/
+    2193,
+/**/
+    2192,
+/**/
+    2191,
+/**/
+    2190,
+/**/
+    2189,
+/**/
+    2188,
+/**/
+    2187,
+/**/
+    2186,
+/**/
+    2185,
+/**/
+    2184,
+/**/
+    2183,
+/**/
+    2182,
+/**/
+    2181,
+/**/
+    2180,
+/**/
+    2179,
+/**/
+    2178,
+/**/
+    2177,
+/**/
+    2176,
+/**/
+    2175,
+/**/
+    2174,
+/**/
+    2173,
+/**/
+    2172,
+/**/
+    2171,
+/**/
+    2170,
+/**/
+    2169,
+/**/
+    2168,
+/**/
+    2167,
+/**/
+    2166,
+/**/
+    2165,
+/**/
+    2164,
+/**/
+    2163,
+/**/
+    2162,
+/**/
+    2161,
+/**/
+    2160,
+/**/
+    2159,
+/**/
+    2158,
+/**/
+    2157,
+/**/
+    2156,
+/**/
+    2155,
+/**/
+    2154,
+/**/
+    2153,
+/**/
+    2152,
+/**/
+    2151,
+/**/
+    2150,
+/**/
+    2149,
+/**/
+    2148,
+/**/
+    2147,
+/**/
+    2146,
+/**/
+    2145,
+/**/
+    2144,
+/**/
+    2143,
+/**/
+    2142,
+/**/
+    2141,
+/**/
+    2140,
+/**/
+    2139,
+/**/
+    2138,
+/**/
+    2137,
+/**/
+    2136,
+/**/
+    2135,
+/**/
+    2134,
+/**/
+    2133,
+/**/
+    2132,
+/**/
+    2131,
+/**/
+    2130,
+/**/
+    2129,
+/**/
+    2128,
 /**/
     2127,
 /**/
@@ -5241,6 +5493,11 @@ list_version(void)
 # else
     msg_puts(_("\nmacOS version w/o darwin feat."));
 # endif
+# if defined(__arm64__)
+    msg_puts(" - arm64");
+# elif defined(__x86_64__)
+    msg_puts(" - x86_64");
+# endif
 #endif
 
 #ifdef VMS
